/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.runtime.assembler.cli;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.repository.layout.ArtifactRepositoryLayout;
import org.apache.maven.artifact.resolver.ArtifactResolutionRequest;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ResolutionErrorHandler;
import org.apache.maven.repository.RepositorySystem;
import org.codehaus.plexus.archiver.UnArchiver;
import org.codehaus.plexus.archiver.manager.ArchiverManager;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.codehaus.plexus.util.FileUtils;
import org.sonatype.runtime.assembler.ArtifactResolver;
import org.sonatype.runtime.descriptor.AssemblyDescriptorArtifact;

@Component( role = RepositorySystemArtifactResolver.class )
public class RepositorySystemArtifactResolver
    implements ArtifactResolver
{

    @Requirement
    private ResolutionErrorHandler resolutionErrorHandler;

    @Requirement
    protected RepositorySystem repositorySystem;

    @Requirement
    protected ArchiverManager archiverManager;

    private final File resolutionDir;
    private ArtifactRepository localRepository;
    private List<ArtifactRepository> remoteRepositories;
    private boolean offline;
    private boolean forceUpdate;

    public RepositorySystemArtifactResolver()
    {
        resolutionDir = FileUtils.createTempFile( "resolver-", "", null );
        resolutionDir.mkdirs();
        // TODO use some logger
        System.err.printf( "Temporary directory: %s\n", resolutionDir.getAbsolutePath() );
    }

    public File resolve( final AssemblyDescriptorArtifact artifact,
                         final boolean unpack )
        throws Exception
    {
        final Artifact requested = repositorySystem.createArtifactWithClassifier(
            artifact.getGroupId(), artifact.getArtifactId(), artifact.getVersion(),
            artifact.getType(), artifact.getClassifier()
        );

        final ArtifactResolutionRequest request = new ArtifactResolutionRequest()
            .setLocalRepository( localRepository )
            .setRemoteRepositories( remoteRepositories )
            .setOffline( offline )
            .setForceUpdate( forceUpdate )
//            .setCache( session.getRepositoryCache() )
//            .setServers( session.getRequest().getServers() )
//            .setMirrors( session.getRequest().getMirrors() )
//            .setProxies( session.getRequest().getProxies() )
            .setArtifact( requested )
            .setResolveTransitively( false );

        final ArtifactResolutionResult result = repositorySystem.resolve( request );
        resolutionErrorHandler.throwErrors( request, result );

        final Artifact resolved = result.getArtifacts().iterator().next();

        final File location;
        if ( unpack )
        {
            // TODO better naming strategy for location dir name?
            location = new File( resolutionDir, resolved.getFile().getName() );
            location.mkdirs();

            final UnArchiver unArchiver = archiverManager.getUnArchiver( resolved.getFile() );
            unArchiver.setSourceFile( resolved.getFile() );
            unArchiver.setDestDirectory( location );
            unArchiver.extract();
        }
        else
        {
            location = resolved.getFile();
        }

        return location;
    }

    public RepositorySystemArtifactResolver setLocalRepository( final String localRepository )
        throws Exception
    {
        if ( localRepository == null )
        {
            this.localRepository = repositorySystem.createDefaultLocalRepository();
        }
        else
        {
            this.localRepository = repositorySystem.createLocalRepository( new File( localRepository ) );
        }
        return this;
    }

    public RepositorySystemArtifactResolver setRemoteRepositories( final String[] remoteRepositories )
        throws Exception
    {
        if ( remoteRepositories == null || remoteRepositories.length == 0 )
        {
            this.remoteRepositories = Arrays.asList( repositorySystem.createDefaultRemoteRepository() );
        }
        else
        {
            this.remoteRepositories = new ArrayList<ArtifactRepository>();
            for ( String remoteRepository : remoteRepositories )
            {
                this.remoteRepositories.add(
                    repositorySystem.createArtifactRepository(
                        remoteRepository,
                        remoteRepository,
                        null,
                        null,
                        null
                    )
                );
            }            
        }
        return this;
    }

    public RepositorySystemArtifactResolver setOffline( final boolean offline )
    {
        this.offline = offline;
        return this;
    }

    public RepositorySystemArtifactResolver setForceUpdate( final boolean forceUpdate )
    {
        this.forceUpdate = forceUpdate;
        return this;
    }
}